/*
 *  Engine/Incpkg.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.IO;
using System.Collections;


namespace Incollector.Engine {
	
	public static class Incpkg {
		
		public struct FInfo {
			public string Name;
			public int    Length;
			
			public FInfo (string name, int length)
			{
				this.Name = name;
				this.Length = length;
			}
		}
		
		
		
		public static void Pack (string Directory, string PackageFileName)
		{
			FileStream fout = new FileStream (PackageFileName, FileMode.Create, FileAccess.Write);
			ArrayList  dirs = GetDirectories (Directory);
			int        i    = 0;
			
			foreach (string dir in dirs) {
				foreach (byte bte in GetFileInfo (Directory, dir))
					fout.WriteByte (bte);
				
				if (i < dirs.Count - 1)
					fout.WriteByte ((byte) '%');
				
				i++;
			}
			
			fout.WriteByte ((byte) '\n');
			
			foreach (string dir in dirs) {
				System.IO.FileStream fread = new FileStream (dir, FileMode.Open, FileAccess.Read);
				int j = 0;
				while (j < fread.Length) {
					fout.WriteByte ((byte) fread.ReadByte ());
					j++;
				}
				
				fread.Close ();
			}
			
			fout.Close ();
		}
		
		
		public static int Unpack (string PackageFileName, string NewDirectory)
		{
			FileStream fread     = new FileStream (PackageFileName, FileMode.Open, FileAccess.Read);
			ArrayList  FilesList = new ArrayList ();
			
			char   chr;
			string head = "";
			
			while ((chr = (char) fread.ReadByte ()) != '\n')
				head += chr;
			
			foreach (string fhead in head.Split ('%')) {
				try {
					string [] file = fhead.Split (':');
					FilesList.Add (new FInfo (file [0], Convert.ToInt32 (file [1])));
				} catch {
					return -1;
				}
			}
			
			try {
				foreach (FInfo file in FilesList) {
					System.IO.Directory.CreateDirectory (Path.Combine (NewDirectory, Path.GetDirectoryName (file.Name)));
					FileStream fwrite = new FileStream (Path.Combine (NewDirectory, file.Name), FileMode.Create, FileAccess.Write);
					
					int cnt = 0;
					while (cnt < file.Length) {
						fwrite.WriteByte ((byte) fread.ReadByte ());
						cnt++;
					}
					
					fwrite.Close ();
				}
			} catch {
				return -1;
			}
			
			fread.Close ();
			return 0;
		}
		
		
		private static string GetFileInfo (string BaseDirectory, string FileName)
		{
			return GetPath (BaseDirectory, FileName) + ":" + (new System.IO.FileInfo (FileName)).Length;
		}
		
		
		private static ArrayList GetDirectories (string Directory)
		{
			DirectoryInfo [] dirs = (new DirectoryInfo (Directory)).GetDirectories ();
			ArrayList        list = new ArrayList ();
			
			foreach (DirectoryInfo dir in dirs)
				list.AddRange (GetDirectories (dir.FullName));
			
			
			FileInfo [] files = (new DirectoryInfo (Directory)).GetFiles ();
			foreach (FileInfo file in files)
				if (file.Name.EndsWith ("~") == false)
					list.Add (file.FullName);
			
			return list;
		}
		
		
		private static string GetPath (string BaseDirectory, string path)
		{
			char sep = '/';
			if ((Environment.OSVersion.Platform == PlatformID.Win32NT) ||
				    	(Environment.OSVersion.Platform == PlatformID.Win32S) ||
					    (Environment.OSVersion.Platform == PlatformID.Win32Windows) ||
					    (Environment.OSVersion.Platform == PlatformID.WinCE))
				sep = '\\';
			
			return path.Replace (BaseDirectory + sep, "");
		}
	}
}
