/*
 *  GUI/EntriesList/EntriesList.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using Incollector.Engine;


namespace Incollector.GUI {

	public abstract class EntriesList : Gtk.VBox {
		
		public enum SortByEnum {
			Category = 0,
			Date     = 1,
			Rating   = 2
		};
		
		
		public EntriesList ()
		{
			Create ();
			this.ShowAll ();
		}
		
		
		// Parameters
		public abstract SortByEnum  SortBy           { get ; set ; }
		public abstract int         NSelectedEntries { get ; }
		public abstract int         NEntries         { get ; }
		public abstract Gtk.Widget  ListWidget       { get ; }
		
		// Signals
		public delegate void SelectionChangedHandler (int NItems);
		public delegate void EntryActivateHandler    (Engine.EntryStruct entry);
		
		public SelectionChangedHandler     SelectionChanged;
		public EntryActivateHandler        EntryActivated;
		public EventHandler                RemoveEntryActivated;
		public EventHandler                AddEntryActivated;
		public Gtk.DragDataReceivedHandler DragDataReceived;
		
		
		// Other functions
		public abstract void Create    ();
		public abstract void Clear     ();
		public abstract void ExpandAll ();
		public abstract void Zoom      (int size); // percents
		
		public abstract ArrayList GetSelectedEntries ();
		public abstract int       GetDroppedEntryID  (Gtk.DragDataReceivedArgs args);
		
		public abstract void AddEntry    (Engine.EntryStruct entry, bool Expand);
		public abstract void ModifyEntry (Engine.EntryStruct NewEntry);
		public abstract void RemoveEntry (Engine.EntryStruct entry);
		public abstract bool Exists      (int EntryID);
		
		public bool Exists (Engine.EntryStruct entry)
		{
			return Exists (entry.id);
		}
	}
	
}
