/*
 *  GUI/EntryDialog.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.Reflection;
using Gdk;
using Gtk;
using Glade;
using Mono.Unix;
using Incollector.Engine;
using Incollector.GUI.Widgets;


namespace Incollector.GUI {
	
	/*
	 * EntryDialog class
	 */
	
	public class EntryDialog {
		private Glade.XML     gui;
		public  Gtk.Dialog    dialog;
		private bool          NewEntry;
		private Engine.EntryStruct vEntry;
		private string        DataType, Content, CurrentDataType;
		private Gtk.Widget    FocusWidget, DefaultFocusWidget;
		
		[Glade.Widget] private Gtk.Viewport  Bg;
		[Glade.Widget] public  Gtk.Label     OkBtnLbl;
		[Glade.Widget] public  Gtk.Image     OkBtnImg;
		[Glade.Widget] private Gtk.VBox      EntryTypeVBox, Page;
		[Glade.Widget] private Gtk.ComboBox  EntryType, Rating;
		[Glade.Widget] private Gtk.Table     TagsTable;
		[Glade.Widget] private Gtk.TextView  Comment;
		[Glade.Widget] private Gtk.Button    OkBtn;
		[Glade.Widget] private TagsEntry     Tags;
		
		

		/* EntryDialog */
		public EntryDialog (Gtk.Window parent, string [] Tags, bool NewEntry)
		{
			CreateEntryDialog (parent, Tags, NewEntry, "", "");
		}
		
		
		/* EntryDialog */
		public EntryDialog (Gtk.Window parent, string [] Tags, bool NewEntry, string DataType, string Content)
		{
			CreateEntryDialog (parent, Tags, NewEntry, DataType, Content);
		}
		
		
		private void CreateEntryDialog (Gtk.Window parent, string [] tags, bool NewEntry, string DataType, string Content)
		{
			this.CurrentDataType = DataType;
			this.DataType = DataType;
			this.Content  = Content;
			
			
			gui = Glade.XML.FromAssembly ("gui.glade", "EntryDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("EntryDialog");
			dialog.TransientFor = parent;
			
			Tags = new TagsEntry (tags);
			TagsTable.Attach (Tags, 0, 1, 0, 1);
			
			Icons.AssignWindow (dialog);
			
			this.NewEntry = NewEntry;
			
			Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (237, 237, 237));
			
			int i = 0;
			foreach (DataTypes.DataType dtp in DataTypes.DataTypes.DataTypesList) {
				EntryType.AppendText (dtp.Title);
				
				if ((DataType != "") && (NewEntry = true) && (dtp.Name == DataType))
					EntryType.Active = i;
				
				i++;
			}
			
			if ((NewEntry) && (DataType == ""))
				if (Engine.Settings.Get ("EntryTypeID") == "")
					EntryType.Active = 1;
				else
					EntryType.Active = Convert.ToInt32 (Engine.Settings.Get ("EntryTypeID"));
			
			Rating.Active = 0;
			
			
			if (NewEntry == false) {
				OkBtnLbl.Text = Catalog.GetString ("OK");
				OkBtnImg.Stock = Gtk.Stock.Ok;
				dialog.Title = Catalog.GetString ("Editing entry");
				EntryTypeVBox.Hide ();
			}
			
			OkBtn.Sensitive = !NewEntry;
			
			if ((NewEntry == true) && (DataType != ""))
				if (Content == "")
					dialog.Focus = DefaultFocusWidget;
				else
					dialog.Focus = FocusWidget;
			
			dialog.Show ();
		}
		
		
		public void SetEntry (Engine.EntryStruct entry)
		{
			vEntry = entry;
			vEntry.tags = new ArrayList (entry.tags);

			Tags.Text      = vEntry.GetTagsString (false);
			Comment.Buffer.Text = vEntry.comment;
			Rating.Active       = vEntry.rating;
			
			Gtk.TreeIter iter;
			int          i = 0;
			for (bool tmp = EntryType.Model.GetIterFirst (out iter); tmp == true; tmp = EntryType.Model.IterNext (ref iter)) {
				if ((string) EntryType.Model.GetValue (iter, 0) == DataTypes.DataTypes.GetDataType (vEntry.type).Title) {
					EntryType.Active = i;
					break;
				}
				
				i++;
			}
			
			dialog.Focus = FocusWidget;
		}
		
		
		public Engine.EntryStruct GetEntry ()
		{
			Gtk.TreeIter iter;
			EntryType.GetActiveIter (out iter);	
			
			if (NewEntry == true) {
				vEntry.date   = DateTime.Today;
				vEntry.tags   = new ArrayList ();
			}
			
			vEntry.comment = Comment.Buffer.Text;
			vEntry.rating  = Rating.Active;
			vEntry.type    = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).Name;
			vEntry.data    = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).GetData ();
			
			vEntry.RemoveAllTags ();
			foreach (string tag in Tags.Text.Split (','))
				if (tag.Trim () != "")
					vEntry.AddTag (tag.Trim ());
			
			return vEntry;
		}
		
		
		public void on_EntryType_changed (object o, EventArgs args)
		{
			if (CurrentDataType != "") {
				int winW, winH;
				dialog.GdkWindow.GetSize (out winW, out winH);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Width", winW);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Height", winH);
			}
			
			CurrentDataType = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).Name;
			
			if (Page.Children.Length > 0)
				Page.Remove (Page.Children [0]);
			
			Gtk.Widget EditWidget;
			if (NewEntry)
				if (DataType != "")
					EditWidget = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).EntryEditWidget (Content);
				else
					EditWidget = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).EntryEditWidget ();
			else
				EditWidget = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).EntryEditWidget (vEntry);
			
			Page.PackStart (EditWidget, true, true, 6);
			FocusWidget = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).FocusWidget;
			DefaultFocusWidget = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).DefaultFocusWidget;
			
			(DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).AddChangedEvent (new EventHandler (on_EntryChanged));
			
			if (NewEntry)
				OkBtn.Sensitive = false;
			
			int width  = Engine.Settings.GetInt ("EntryDialog." + CurrentDataType + ".Width");
			int height = Engine.Settings.GetInt ("EntryDialog." + CurrentDataType + ".Height");
			if ((width != -1) && (height != -1))
				dialog.Resize (width, height);
			else {
				dialog.Resizable = false;
				Tools.Repaint ();
				dialog.Resizable = true;
			}
		}
		
		
		public void on_SomeEntry_activate (object o, EventArgs args)
		{
			dialog.Respond (Gtk.ResponseType.Ok);
		}
		
		
		public void on_EntryDialog_response (object o, ResponseArgs args)
		{
			if (args.ResponseId == ResponseType.Ok) {
				Engine.Settings.Set ("EntryTypeID", EntryType.Active.ToString ());
				
				int winW, winH;
				dialog.GdkWindow.GetSize (out winW, out winH);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Width", winW);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Height", winH);
			}
		}
		
		
		private void on_EntryChanged (object o, EventArgs args)
		{
			OkBtn.Sensitive = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).IsFormFilledIn ();
		}
	}
}
