/*
 *  GUI/MainWindow.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



// if beta version
//#define beta
// ---------------



using System;
using System.Reflection;
using System.Collections;
using System.Threading;
using Incollector.GUI.Widgets;
using Gdk;
using Gtk;
using GLib;
using Mono.Unix;
using Incollector.Engine;
using Glade;



namespace Incollector.GUI {
	
	/*
	 * MainWindow class
	 */
	
	public class MainWindow {
				
		private enum ShowEntriesEnum {
			Today           = 0,
			Last7Days       = 1,
			SelectedDay     = 2,
			SelectedDays    = 3,
			AllEntries      = 4,
			ImportedEntries = 5
		};
		
		private enum PagesEnum {
			Main        = 0,
			Message     = 1,
			BetaVersion = 2
		};

		private enum EntriesListNotebookEnum {
			Normal  = 0,
			Message = 1
		};
		
		private enum PreviewPageEnum {
			Preview      = 0,
			SelectedInfo = 1
		};
		
		private struct BrowsingSettings {
			public static int       show;    // ShowEntries active
			public static ArrayList tags;    // ID of inactive tags
			public static bool      restore; // should settings be restored ?
		}
		
		private Glade.XML   gui;
		private DateTime    SelectedDay, SelectedDaysFrom, SelectedDaysTo;
		public  Gtk.Window  window;
		private EntriesList Entries;
		private TagsList    Tags;
		private bool        Init, IsTrayIcon, NoReloadEntries;
		public  Gtk.Menu    MainMenuIncollector, MainMenuEntries, MainMenuView, MainMenuHelp;
		private int         PreviewEntryID;

		
		[Glade.Widget] private Gtk.VBox             EntriesVBox, SearchBox, TagsBox, PreviewBox, CommentVBox, PreviewVBox;
		[Glade.Widget] private Gtk.ComboBox         ShowEntries, SortBy;
		[Glade.Widget] private Gtk.Notebook         PreviewPage, Pages, EntriesListNotebook;
		[Glade.Widget] private Gtk.Entry            SearchKeyword;
		[Glade.Widget] private Gtk.Button           FindBtn;
		[Glade.Widget] private Gtk.ToolButton       RemoveBtn, AcceptBtn;
		[Glade.Widget] private Gtk.MenuToolButton   AddBtn;
		[Glade.Widget] private Gtk.ToggleToolButton PreviewTggBtn;
		[Glade.Widget] private Gtk.ToggleButton     FindTggBtn;
		[Glade.Widget] private Gtk.Label            MessageA, MessageB, EntriesMsg, SelectedInfo, PreviewComment, ZoomPercent;
		[Glade.Widget] private Gtk.Viewport         SearchResultsBg, ImportedEntriesBg, SearchBg, TagsViewport;
		[Glade.Widget] public  Gtk.MenuBar          MainMenu;
		[Glade.Widget] private Gtk.MenuItem         IncollectorMenu, EntriesMenu, ViewMenu, HelpMenu;
		[Glade.Widget] private Gtk.Toolbar          MainToolbar;
		[Glade.Widget] private Gtk.HSeparator       PreviewSeparator1;
		[Glade.Widget] private Gtk.CheckMenuItem    show_toolbar, show_tags_sidebar, show_preview;
		[Glade.Widget] private Gtk.VPaned           EntriesVPaned;
		[Glade.Widget] private Gtk.HPaned           MainPaned;
		
		
		
		/* Constructor */
		public MainWindow ()
		{
			Init = true;
			
			DB.Loaded  += new DB.LoadHandler   (OnDBLoaded);
			DB.Added   += new DB.AddHandler    (OnDBAddedEntry);
			DB.Updated += new DB.UpdateHandler (OnDBUpdatedEntry);
			DB.Removed += new DB.RemoveHandler (OnDBRemovedEntry);
			
			
			gui = Glade.XML.FromAssembly ("gui.glade", "MainWindow", null);
			gui.Autoconnect (this);
			window = (Gtk.Window) gui.GetWidget ("MainWindow");
			Icons.AssignWindow (window);
			Icons.AssignImage (Icons.LOGO_BIG, "IncStartLogo", gui);
			Icons.AssignMenuItem (Icons.IMPORT_MENU, "import_entries", gui);
			Icons.AssignMenuItem (Icons.EXPORT_MENU, "export_entries", gui);
			
			// Window location and size from settings
			if ((Engine.Settings.GetInt ("MainWindowPositionX") != -1) && (Engine.Settings.GetInt ("MainWindowPositionY") != -1))
				window.Move (Engine.Settings.GetInt ("MainWindowPositionX"), Engine.Settings.GetInt ("MainWindowPositionY"));
			
			if ((Engine.Settings.GetInt ("MainWindowHeight") != -1) &&
				(Engine.Settings.GetInt ("MainWindowWidth") != -1))
				window.SetDefaultSize (Engine.Settings.GetInt ("MainWindowWidth"),
										Engine.Settings.GetInt ("MainWindowHeight"));
			
			if (Engine.Settings.GetBoolDF ("MainWindowMaximized") == true)
				window.Maximize ();
			
			
			Colors.Modify (Colors.INFO, "MsgBg", gui);
			
			Colors.Modify (Colors.WHITE, "PreviewBg", gui);
			Colors.Modify (Colors.WHITE, "EntriesMsgBg", gui);
			Pages.ShowTabs = false;
			
			window.Show ();
			ShowWindowMessage (Catalog.GetString ("Please wait"),
							   Catalog.GetString ("Loading database"));
			
			// Colors from settings
			if (Engine.Settings.GetBool ("UseDefaultColors") == false) {
				Colors.Modify (Engine.Settings.GetColor ("SearchPanelColor"), "SearchResultsBg", gui);
				Colors.Modify (Engine.Settings.GetColor ("SearchPanelColor"), "SearchBg", gui);
				Colors.Modify (Engine.Settings.GetColor ("ImportedEntriesPanelColor"), "ImportedEntriesBg", gui);
			}
			
			// menus
			MainMenuIncollector = IncollectorMenu.Submenu as Gtk.Menu;
			MainMenuEntries     = EntriesMenu.Submenu as Gtk.Menu;
			MainMenuView        = ViewMenu.Submenu as Gtk.Menu;
			MainMenuHelp        = HelpMenu.Submenu as Gtk.Menu;
			
			// TagsList widget
			Tags = new TagsList ();
			Tags.TagsActivityChanged += new TagsList.TagActivityChangedHandler (on_TagsActivity_changed);
			Tags.RemoveTagActivate   += new TagsList.RemoveTagActivateHandler  (on_RemoveTag_activate);
			Tags.RenameTagActivate   += new TagsList.RenameTagActivateHandler  (on_RenameTag_activate);
			TagsBox.Add (Tags);
			
			// Entries widget
			Entries = new ListView ();
			Entries.EntryActivated       += new EntriesList.EntryActivateHandler (on_Entries_entry_activated);
			Entries.SelectionChanged     += new EntriesList.SelectionChangedHandler (on_Entries_selection_changed_handler);
			Entries.DragDataReceived     += new Gtk.DragDataReceivedHandler (on_Entries_data_received);
			Entries.AddEntryActivated    += new EventHandler (on_AddBtn_clicked);
			Entries.RemoveEntryActivated += new EventHandler (on_RemoveBtn_clicked);
			EntriesVBox.Add (Entries);
			
			
			// SortBy active item
			if (Engine.Settings.Get ("SortByID") == "")
				SortBy.Active = (int) EntriesList.SortByEnum.Date;
			else
				SortBy.Active = Convert.ToInt32 (Engine.Settings.Get ("SortByID"));
			
			// ShowEntries active item
			if (Engine.Settings.Get ("ShowEntriesID") == "")
				ShowEntries.Active = (int) ShowEntriesEnum.Last7Days;
			else
				ShowEntries.Active = Convert.ToInt32 (Engine.Settings.Get ("ShowEntriesID"));
			
			
			// Dates
			try {
				string [] arr = Engine.Settings.Get ("ShowEntryDay").Split ('-');
				SelectedDay = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
			} catch {
				SelectedDay = DateTime.Today;
			}
			
			try {
				string [] arr = Engine.Settings.Get ("ShowEntryDaysFrom").Split ('-');
				SelectedDaysFrom = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
			} catch {
				SelectedDaysFrom = DateTime.Today;
			}
			
			try {
				string [] arr = Engine.Settings.Get ("ShowEntryDaysTo").Split ('-');
				SelectedDaysTo = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
			} catch {
				SelectedDaysTo = DateTime.Today;
			}
			
			
			// Showing Preview
			show_preview.Active       = Engine.Settings.GetBool ("ShowPreview");
			PreviewTggBtn.Active      = Engine.Settings.GetBool ("ShowPreview");
			PreviewSeparator1.Visible = Engine.Settings.GetBool ("ShowPreview");
			PreviewVBox.Visible       = Engine.Settings.GetBool ("ShowPreview");
			
			if (Engine.Settings.GetInt ("PreviewPosition") != -1)
				EntriesVPaned.Position = Engine.Settings.GetInt ("PreviewPosition");
			
			if (Engine.Settings.GetInt ("MainPanedPosition") != -1)
				MainPaned.Position = Engine.Settings.GetInt ("MainPanedPosition");
			
			// Showing main toolbar
			show_toolbar.Active = Engine.Settings.GetBool ("ShowMainToolbar");
			MainToolbar.Visible = Engine.Settings.GetBool ("ShowMainToolbar");
			
			// Showing Sidebar
			show_tags_sidebar.Active = Engine.Settings.GetBool ("ShowSidebar");
			TagsViewport.Visible = Engine.Settings.GetBool ("ShowSidebar");
			
			DataTypes.DataTypes.Create ();
			
			// Tray icon
			Incollector.Widgets.GTrayIcon icon = new Incollector.Widgets.GTrayIcon (
														"Incollector", window);
			IsTrayIcon = icon.IsTrayIcon;
			icon.AddEntryActivated = new EventHandler (TrayIcon_on_AddEntry_activated);
			icon.ExitActivated     = new EventHandler (TrayIcon_on_Exit_activated);
			icon.DragDataReceived  = new Gtk.DragDataReceivedHandler (TrayIcon_on_DragData_received);
			
			
			// Database 
			if (DB.Load () == false)
				NoEntriesMessage ();
			
			Tags.LoadTagsActive (new ArrayList (Engine.Settings.Get ("DisabledTags").Split (',')));
			
			Init = false;
			
			// Auto save - every 10 seconds
			GLib.Timeout.Add (10000, Autosave);

			
			// Menu
			Gtk.Menu AddEntryMenu = new Gtk.Menu ();
			foreach (DataTypes.DataType dtp in DataTypes.DataTypes.DataTypesList) {
				Gtk.MenuItem item = new Gtk.MenuItem (dtp.MenuText);
				item.Name = dtp.Name;
				AddEntryMenu.Add (item);
				item.Activated += new EventHandler (on_AddSpecifiedEntry_activated);
			}
			
			AddEntryMenu.ShowAll ();
			AddBtn.Menu = AddEntryMenu;
			
			
			// Zoom
			int zoom = Engine.Settings.GetInt ("zoom");
			if (zoom == -1)
				zoom = 100;
			
			Entries.Zoom (zoom);
			ZoomPercent.Text = zoom.ToString () + "%";
			
			
			HideWindowMessage ();
			window.Focus = Entries.ListWidget;
		}
		
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   Tagslist
		//
		////////////////////////////////////////////////////////////////////////
		
		private void ReloadTags ()
		{
			Tags.Clear ();
			
			foreach (Engine.EntryStruct ent in Engine.DB.Data)
				Tags.AddTags (ent.tags);
		}
		
		
		public void on_TagsActivity_changed ()
		{
			if (Init == false)
				ReloadEntries ();
		}
		
		
		public void on_RemoveTag_activate (string tag)
		{
			SetSensitive (false);
			
			int cnt = DB.Data.Count;
			for (int i = 0; i < cnt; i++) {
				Engine.Entry ent = new Engine.Entry ((EntryStruct) DB.Data [i]);
				
				if (ent.TagExists (tag))
					ent.RemoveTag (tag);
				
				ent.Save ();
			}
			
			
			SetSensitive (true);
		}
		
		
		public void on_RenameTag_activate (string tag)
		{
			SetSensitive (false);
			
			TagDialog dlg = new TagDialog (tag);
			
			if (dlg.dialog.Run () == (int) Gtk.ResponseType.Ok) {
				int cnt = DB.Data.Count;
				for (int i = 0; i < cnt; i++) {
					Engine.Entry ent = new Engine.Entry ((EntryStruct) DB.Data [i]);
					
					if (ent.TagExists (tag))
						ent.ChangeTag (tag, dlg.TagName.Text);
					
					ent.Save ();
				}
			}
			
			dlg.dialog.Destroy ();
			SetSensitive (true);
		}
		
		
		/// <summary>Shows all the tags</summary>
		public void ShowAllTags ()
		{
			Tags.EnableAllTags ();
			
			if (NoReloadEntries == false)
				ReloadEntries ();
		}
		
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   Entries list
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void on_Entries_entry_activated (Engine.EntryStruct entry)
		{
			SetSensitive (false);
			
			EntryDialog dlg = new EntryDialog (window, Tags.Tags, false);
			dlg.SetEntry (entry);
			
			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				DB.ChangeEntry (dlg.GetEntry ());
			
			dlg.dialog.Destroy ();
			PreviewNotebookLoad (true);
			SetSensitive (true);
		}
		
		
		protected void on_Entries_selection_changed_handler (int NItems)
		{
			RemoveBtn.Sensitive = NItems > 0;
			AcceptBtn.Sensitive = NItems > 0;
			PreviewNotebookLoad (false);
		}
		
		
		protected void on_Entries_data_received (object o, DragDataReceivedArgs args)
		{
			if (args.SelectionData.Text == null)
				return;
			
			if (args.SelectionData.Text.StartsWith ("incollector:tag")) { // if tag
				int EntryID = Entries.GetDroppedEntryID (args);
				if (EntryID == -1)
					return;
				
				Engine.EntryStruct entry = DB.GetEntryByID (EntryID);
				Engine.EntryStruct NewEntry = entry;
				NewEntry.tags = new ArrayList (entry.tags);
				
				NewEntry.AddTag (Tags.GetDraggingTag (args));
				DB.ChangeEntry (NewEntry);
			} else {
				string url = new System.Text.StringBuilder (System.Text.Encoding.UTF8.GetString (args.SelectionData.Data)).ToString ();
				
				try {
					Uri addr = new Uri (url);
					
					url = url.Replace ("file:///", "").Trim ();
					
					if (! Engine.FilesPaths.IsWinSystem ())
						url = "/" + url;
					else {
						url = url.Replace (Convert.ToChar (13), '*');
						url = url.Replace (Convert.ToChar (10), '*');
						url = url.Replace ('/', '\\');
						url = url.Replace ("*", "");
					}
				
					if ((addr.IsFile == true) && (System.IO.File.Exists (url.ToString ()))) { // if file
						// checking file extension
						if (System.IO.Path.GetExtension (url) != ".ied") {
							window.Sensitive = false;
							MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Error,
									ButtonsType.Ok, Catalog.GetString ("The file has an invalid extension."));
							msg.Run ();
							msg.Destroy ();
							window.Sensitive = true;
							return;
						}
						
						on_import_entries_activate (url, null);
					} else
						if ((addr.ToString ().StartsWith ("http")) || (addr.ToString ().StartsWith ("ftp")))
							AddEntry ("webaddress", addr.ToString ());
				} catch {
					return;
				}
			}
		}
		
		
		/* Can entry be showed ? (checks which days can be showed [date]) */
		private bool EntryCheckDay (Engine.EntryStruct ent)
		{
			switch (ShowEntries.Active) {
				case (int) ShowEntriesEnum.Today:
				return (ent.date.Day == DateTime.Today.Day) && (ent.date.Month == DateTime.Today.Month) &&
				        (ent.date.Year == DateTime.Today.Year);
				
				case (int) ShowEntriesEnum.Last7Days:
					DateTime begin = DateTime.Today.AddDays (-7);
				return (ent.date >= begin) && (ent.date <= DateTime.Today);
				
				case (int) ShowEntriesEnum.SelectedDay:
				return ent.date == SelectedDay;
				
				case (int) ShowEntriesEnum.SelectedDays:
				return (ent.date >= SelectedDaysFrom) && (ent.date <= SelectedDaysTo);
				
				case (int) ShowEntriesEnum.AllEntries:
				return true;
			}
			
			
			return true;
		}
		
		
		
		/* Can entry be showed ? (search) */
		private bool EntryCheckSearch (Engine.EntryStruct ent)
		{
			if (SearchKeyword.Text != "")
				return DataTypes.DataTypes.GetDataType (ent.type).Search (ent, SearchKeyword.Text) ||
						Engine.Text.Search (SearchKeyword.Text, ent.comment);
			
			return true;
		}
		
		
		/* Can entry be showed ? (checks what tags can entries be showed with) */
		private bool EntryCheckTags (Engine.EntryStruct entry, ArrayList EnabledTags)
		{
			if (entry.tags.Count == 0)
				return true;
			
			foreach (string tag in entry.tags)
				foreach (string stag in EnabledTags)
					if (stag == tag)
						return true;
			
			return false;
		}
		
		
		/* Can entry be showed ? (show imported) */
		private bool EntryCheckImported (Engine.EntryStruct ent)
		{
			if (ShowEntries.Active == (int) ShowEntriesEnum.ImportedEntries)
				return ent.imported && ent.accepted == false;
			else
				return (! ent.imported) || (ent.imported && ent.accepted);
		}
		
		
		/// <summary>Gets enabled tags list. Warning ! It should be use only if Init == true.</summary>
		protected ArrayList GetEnabledTagsList ()
		{
			ArrayList list     = new ArrayList (Tags.EnabledTagsList ());
			ArrayList disabled = new ArrayList (Engine.Settings.Get ("DisabledTags").Split (','));
			
			foreach (string tag in disabled)
				list.Remove (Text.PrepareText2 (tag));
			
			return list;
		}
		
		
		private void ReloadEntries ()
		{
			if (NoReloadEntries == true)
				return;
			
			Entries.Clear ();
			
			ArrayList EnabledTags;
			if (Init == false)
				EnabledTags = Tags.EnabledTagsList ();
			else
				EnabledTags = GetEnabledTagsList ();
			
			foreach (Engine.EntryStruct ent in Engine.DB.Data)
				if ((EntryCheckDay (ent)) && (EntryCheckSearch (ent)) &&
							(EntryCheckTags (ent, EnabledTags)) && (EntryCheckImported (ent)))
					Entries.AddEntry (ent, false);
			
			Entries.ExpandAll ();
			NoEntriesMessage ();
		}
		
		
		/// <summary>Loads entry to PreviewNotebook.</summary>
		private void PreviewNotebookLoad (bool ForceLoad)
		{
			if (Entries.NSelectedEntries == 1) {
				Engine.EntryStruct ent = Engine.DB.GetEntryByID ((int) Entries.GetSelectedEntries () [0]);
				if ((PreviewEntryID == ent.id) && (ForceLoad == false))
					return;
				
				if (PreviewBox.Children.Length > 0)
					PreviewBox.Remove (PreviewBox.Children [0]);
				
				CommentVBox.Hide ();
				SelectedInfo.Markup = "<b>" + Catalog.GetString ("Loading preview...") + "</b>";
				PreviewPage.Page = (int) PreviewPageEnum.SelectedInfo;
				Tools.Repaint ();
				
				PreviewBox.PackStart (DataTypes.DataTypes.GetDataType (ent.type).EntryPreviewWidget (ent), false, false, 6);
				
				if (ent.comment != "") {
					PreviewComment.Text = ent.comment;
					CommentVBox.Show ();
				}
				
				PreviewEntryID = ent.id;
				PreviewPage.Page = (int) PreviewPageEnum.Preview;
			} else {
				if (PreviewBox.Children.Length > 0)
					PreviewBox.Remove (PreviewBox.Children [0]);
				
				SelectedInfo.Markup = "<b>" + Catalog.GetString ("Preview unavailable") + "</b>";
				PreviewPage.Page = (int) PreviewPageEnum.SelectedInfo;
				CommentVBox.Hide ();
				
				PreviewEntryID = -1;
			}
		}
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   DB Callbacs
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void OnDBLoaded (bool LockGUI)
		{
			ReloadTags ();
			ReloadEntries ();
		}
		
		
		protected void OnDBAddedEntry (Engine.EntryStruct entry, bool LockGUI)
		{
			Tags.AddTags (entry.tags);
			ArrayList EnabledTags = Tags.EnabledTagsList ();
			
			if ((EntryCheckDay (entry)) && (EntryCheckSearch (entry)) &&
					(EntryCheckTags (entry, EnabledTags)) && (EntryCheckImported (entry)))
				Entries.AddEntry (entry, true);
		}
		
		
		protected void OnDBUpdatedEntry (Engine.EntryStruct entry, Engine.EntryStruct OldEntry, bool LockGUI)
		{
			Tags.ApplyDiff (new ArrayList (OldEntry.tags), new ArrayList (entry.tags));
			ArrayList EnabledTags = Tags.EnabledTagsList ();
			
			if ((EntryCheckDay (entry)) && (EntryCheckSearch (entry)) &&
					(EntryCheckTags (entry, EnabledTags)) && (EntryCheckImported (entry)))
				Entries.ModifyEntry (entry);
			else
				Entries.RemoveEntry (entry);
		}
		
		
		protected void OnDBRemovedEntry (Engine.EntryStruct entry, bool LockGUI)
		{
			Entries.RemoveEntry (entry);
			Tags.RemoveTags (entry.tags);
		}
		
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   widgets callbacks
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void on_preferences_activate (object o, EventArgs args)
		{
			PreferencesDialog dlg = new PreferencesDialog (window);
			SetSensitive (false);

			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				ReloadEntries ();

				if (Engine.Settings.GetBool ("UseDefaultColors") == false) {
					SearchResultsBg.ModifyBg   (Gtk.StateType.Normal, Engine.Settings.GetColor ("SearchPanelColor"));
					SearchBg.ModifyBg          (Gtk.StateType.Normal, Engine.Settings.GetColor ("SearchPanelColor"));
					ImportedEntriesBg.ModifyBg (Gtk.StateType.Normal, Engine.Settings.GetColor ("ImportedEntriesPanelColor"));
				} else {
					SearchResultsBg.ModifyBg   (Gtk.StateType.Normal);
					SearchBg.ModifyBg          (Gtk.StateType.Normal);
					ImportedEntriesBg.ModifyBg (Gtk.StateType.Normal);
				}
			}

			dlg.dialog.Destroy ();
			SetSensitive (true);
		}
		
		
		protected void on_plugins_activate (object o, EventArgs args)
		{
			PluginsDialog dlg = new PluginsDialog (window);
			SetSensitive (false);

			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				ReloadEntries ();

			dlg.dialog.Destroy ();
			SetSensitive (true);
		}
		
		
		protected void on_hide_window_activate (object o, EventArgs args)
		{
			HideMainWindow ();
		}
		
		
		protected void on_exit_activate (object o, EventArgs args)
		{
			ExitApplication ();
		}
		
		
		protected bool Autosave ()
		{
			GLib.Timeout.Add (10000, Autosave);
			
			SaveAllSettings (true);
			FilesPaths.MoveAutosaveFiles ();
			
			return false;
		}
		
		
		public void SaveAllSettings ()
		{
			SaveAllSettings (false);
		}
		
		
		public void SaveAllSettings (bool Autosave)
		{
			// Settings
			Engine.Settings.Set     ("SortByID", SortBy.Active.ToString ());
			Engine.Settings.Set     ("SearchInID", ShowEntries.Active.ToString ());
			Engine.Settings.Set     ("ShowEntriesID", ShowEntries.Active.ToString ());
			Engine.Settings.SetBool ("ShowPreview", PreviewSeparator1.Visible);
			Engine.Settings.SetBool ("ShowPreview", PreviewVBox.Visible);
			Engine.Settings.SetInt  ("PreviewPosition", EntriesVPaned.Position);
			Engine.Settings.SetInt  ("MainPanedPosition", MainPaned.Position);
			Engine.Settings.SetBool ("ShowMainToolbar", MainToolbar.Visible);
			Engine.Settings.SetBool ("ShowSidebar", TagsViewport.Visible);
			
			int winX, winY;
			window.GetPosition (out winX, out winY);
			Engine.Settings.SetInt ("MainWindowPositionX", winX);
			Engine.Settings.SetInt ("MainWindowPositionY", winY);
			
			int winW, winH;
			window.GdkWindow.GetSize (out winW, out winH);
			Engine.Settings.SetInt ("MainWindowWidth", winW);
			Engine.Settings.SetInt ("MainWindowHeight", winH);
			
			Engine.Settings.SetBool ("MainWindowMaximized", window.GdkWindow.State == Gdk.WindowState.Maximized);
			
			
			string DisabledTags = "";
			foreach (string tag in Tags.DisabledTagsList ())
				DisabledTags += Text.PrepareText (tag, false) + ",";
			Engine.Settings.Set ("DisabledTags", DisabledTags);
			
			Engine.Settings.Save (Autosave);
			
			
			// DB
			Engine.DB.Save (Autosave);
		}
		
		
		public void ExitApplication ()
		{
			ShowWindowMessage (Catalog.GetString ("Please wait"),
								Catalog.GetString ("Saving database"));
			SaveAllSettings ();
			
			Application.Quit ();
		}
		
		
		protected void on_add_activate (object o, EventArgs args)
		{
			AddEntry ();
		}
		
		
		protected void on_search_activate (object o, EventArgs args)
		{
			if (SearchBox.Visible)
				window.Focus = SearchKeyword;
			else
				FindTggBtn.Active = true;
		}
		
		
		public void on_import_entries_activate (object o, EventArgs args)
		{
			SetSensitive (false);
			
			string FileName = "";
			if (o.GetType () == typeof (string))
				FileName = o as string;
			
			ImportingEntriesDialog dlg = new ImportingEntriesDialog (window, FileName);
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				dlg.dialog.Destroy ();

				ShowEntriesMessage (Catalog.GetString ("Please wait"), Catalog.GetString ("Importing entries"));
				
				EntriesListNotebook.Page = (int) EntriesListNotebookEnum.Message;
				Tools.Repaint ();
				
				
				if (Engine.DB.ImportEntries (dlg.FileName.Text, dlg.ImportDates.Active,
						dlg.ImportRating.Active, dlg.ImportTags.Active) == false) {
					EntriesListNotebook.Page = (int) EntriesListNotebookEnum.Normal;
					Tools.Repaint ();
					
					MessageDialog msg2 = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Error,
							ButtonsType.Ok, Catalog.GetString ("The file is broken. Entries have not been imported."));
					msg2.Run ();
					msg2.Destroy ();
					window.Sensitive = true;
					return;
				}
				

				ReloadEntries ();
				
				ResetBrowsingSettings (true);
				ImportedEntriesBg.Show ();

				window.Sensitive = false;
			} else
				dlg.dialog.Destroy ();
			
			SetSensitive (true);
		}
		
		
		protected void on_export_entries_activate (object o, EventArgs args)
		{
			SetSensitive (false);

			ExportingEntriesDialog dlg = new ExportingEntriesDialog (window,
							Entries.NSelectedEntries >= 1);
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				dlg.dialog.Destroy ();

				ShowEntriesMessage (Catalog.GetString ("Please wait"), Catalog.GetString ("Exporting entries"));
				
				if (dlg.ExportType.Active == 0) // if selected entries
					Engine.DB.ExportEntries (Entries.GetSelectedEntries (), dlg.FileName.Text);
				else
					Engine.DB.ExportEntries (new ArrayList (), dlg.FileName.Text);
				
				HideEntriesMessage ();
				
				
				Gtk.MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Info,
							ButtonsType.Ok, Catalog.GetString ("Entries have been exported !"));
				msg.Run ();
				msg.Destroy ();
			} else
				dlg.dialog.Destroy ();
			
			SetSensitive (true);
		}
		
		
		protected void on_show_toolbar_activate (object o, EventArgs args)
		{
			if ((o as CheckMenuItem).Active)
				MainToolbar.Show ();
			else
				MainToolbar.Hide ();
		}
		
		
		protected void on_show_tags_sidebar_activate (object o, EventArgs args)
		{
			if ((o as CheckMenuItem).Active)
				TagsViewport.Show ();
			else
				TagsViewport.Hide ();
		}
		
		
		protected void on_show_preview_activate (object o, EventArgs args)
		{
			PreviewTggBtn.Active = (o as CheckMenuItem).Active;
			
			if ((o as CheckMenuItem).Active) {
				PreviewSeparator1.Show ();
				PreviewVBox.Show ();
			} else {
				PreviewSeparator1.Hide ();
				PreviewVBox.Hide ();
			}
		}
		
		
		protected void on_about_activate (object o, EventArgs args)
		{
			SetSensitive (false);
			
			AboutWindow wdw = new AboutWindow (this.window);
			wdw.dialog.Run ();
			wdw.dialog.Destroy ();
			
			SetSensitive (true);
		}
		
		
		/* MainWindow delete */
		protected void on_MainWindow_delete_event (object o, DeleteEventArgs args)
		{
			if ((Engine.Settings.GetBool ("MinimizeToTray") == false) || (IsTrayIcon == false)) {
				ExitApplication ();
				args.RetVal = true;
			} else {
				HideMainWindow ();
				args.RetVal = true;
			}
		}
		
		
		private void HideMainWindow ()
		{
			if (Engine.Settings.GetBool ("CloseHideMsg") == true) {
				TrayMessage msg = new TrayMessage (MsgType.Info, true, null, null,
									Catalog.GetString ("Incollector window has been hidden"),
									Catalog.GetString ("To restore window click Incollector icon in your notification area."));
				msg.Show ();
				Engine.Settings.SetBool ("CloseHideMsg", false);
			}
			
			window.Hide ();
		}
		
		
		/* AddBtn clicked */
		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			AddEntry ();
		}
		
		
		protected void on_AddSpecifiedEntry_activated (object o, EventArgs args)
		{
			AddEntry ((o as MenuItem).Name, "");
		}
		
		
		private void AddEntry ()
		{
			AddEntry ("", "");
		}
		
		
		private void AddEntry (string selected, string content)
		{
			SetSensitive (false);
			
			EntryDialog dlg = new EntryDialog (window, Tags.Tags, true, selected, content);
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				DB.AddEntry (dlg.GetEntry ());
				
				Engine.EntryStruct entry = dlg.GetEntry ();
				if (! ((EntryCheckDay (entry)) && (EntryCheckSearch (entry)) &&
						(EntryCheckTags (entry, Tags.EnabledTagsList ())) && (EntryCheckImported (entry)))) {
					
					Gtk.MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Warning,
							ButtonsType.Ok, Catalog.GetString ("Entry has been added but it couldn't be showed.\nTo see the entry change browsing settings."));
					msg.Run ();
					msg.Destroy ();
				}
			}
			
			dlg.dialog.Destroy ();
			
			NoEntriesMessage ();
			SetSensitive (true);
		}
		
		
		/* RemoveBtn clicked */
		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			ArrayList selected = Entries.GetSelectedEntries ();
			
			if (selected.Count >= 1) {
				SetSensitive (false);
				
				Gtk.MessageDialog msg;
				if (selected.Count == 1)
					msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, Catalog.GetString ("Are you really sure that you want to remove selected entry ?"));
				else
					msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, Catalog.GetString ("Are you really sure that you want to remove selected entries ?"));
				
				if (ResponseType.Yes == (ResponseType) msg.Run ())
					foreach (int EntryID in selected)
						DB.RemoveEntry (DB.GetEntryByID (EntryID));
				
				msg.Destroy ();
				RemoveBtn.Sensitive = false;
				AcceptBtn.Sensitive = false;
				
				SetSensitive (true);
			}
			
			NoEntriesMessage ();
		}
		
		
		protected void on_AcceptBtn_clicked (object o, EventArgs args)
		{
			ArrayList selected = Entries.GetSelectedEntries ();
			if (selected.Count == 0)
				return;
			
			
			SetSensitive (false);
			DB.AcceptEntries (selected);
			ReloadEntries ();
			SetSensitive (true);
			
			NoEntriesMessage ();
		}
		
		
		/* FindTggBtn toggled */
		protected void on_FindTggBtn_toggled (object o, EventArgs args)
		{
			if (FindTggBtn.Active) {
				SearchBox.Show ();
				window.Focus = SearchKeyword;
			} else {
				SearchBox.Hide ();
				SearchResultsBg.Hide ();
				
				if (SearchKeyword.Text != null) {
					SearchKeyword.Text = "";
					RestoreBrowsingSettings ();
				}
			}
		}
		
		
		/* PreviewTggBtn clicked */
		protected void on_PreviewTggBtn_toggled (object o, EventArgs args)
		{
			show_preview.Active = PreviewTggBtn.Active;
			if (PreviewTggBtn.Active)
				PreviewNotebookLoad (true);
		}
		
		
		/* SelectedDay widget changed */
		protected void on_SelectedDay_changed (object o, EventArgs args)
		{
			if (Init == false)
				ReloadEntries ();
		}
		
		
		/* SelectedDaysFrom widget changed */
		protected void on_SelectedDaysFrom_changed (object o, EventArgs args)
		{
			if (Init == false)
				ReloadEntries ();
		}
		
		
		/* SelectedDaysTo widget changed */
		protected void on_SelectedDaysTo_changed (object o, EventArgs args)
		{
			if (Init == false)
				ReloadEntries ();
		}
		
		
		protected void on_SortBy_changed (object o, EventArgs args)
		{
			switch ((o as ComboBox).Active) {
				case (int) EntriesList.SortByEnum.Category:
					Entries.SortBy = EntriesList.SortByEnum.Category;
				break;
				case (int) EntriesList.SortByEnum.Date:
					Entries.SortBy = EntriesList.SortByEnum.Date;
				break;
				case (int) EntriesList.SortByEnum.Rating:
					Entries.SortBy = EntriesList.SortByEnum.Rating;
				break;
			}
			
			if (Init == false)
				ReloadEntries ();
		}
		
		
		protected void on_ShowEntries_changed (object o, EventArgs args)
		{
			SetSensitive (false);
			switch (ShowEntries.Active) {
				case (int) ShowEntriesEnum.SelectedDay:
					if (Init == false) {
						SelectDateDialog dlg = new SelectDateDialog (window, false); 
						if (dlg.dialog.Run () == (int) ResponseType.Ok) {
							this.SelectedDay = dlg.SelectedDay.GetDate ();
							Engine.Settings.Set ("ShowEntryDay", SelectedDay.ToString ("yyyy-MM-dd"));
						}
						
						dlg.dialog.Destroy ();
					}
				break;
				
				case (int) ShowEntriesEnum.SelectedDays:
					if (Init == false) {
						SelectDateDialog dlg2 = new SelectDateDialog (window, true); 
						if (dlg2.dialog.Run () == (int) ResponseType.Ok) {
							this.SelectedDaysFrom = dlg2.SelectedDaysFrom.GetDate ();
							this.SelectedDaysTo   = dlg2.SelectedDaysTo.GetDate ();
							
							Engine.Settings.Set ("ShowEntryDaysFrom", SelectedDaysFrom.ToString ("yyyy-MM-dd"));
							Engine.Settings.Set ("ShowEntryDaysTo", SelectedDaysTo.ToString ("yyyy-MM-dd"));
						}
						
						dlg2.dialog.Destroy ();
					}
				break;
			}
			
			AcceptBtn.Visible = ShowEntries.Active == (int) ShowEntriesEnum.ImportedEntries;
			ImportedEntriesBg.Visible = false;

			if (Init == false)
				ReloadEntries ();
			
			
			SetSensitive (true);
		}
		
		
		/* SearchKeyword changed */
		protected void on_SearchKeyword_changed (object o, EventArgs args)
		{
			if (SearchKeyword.Text != "")
				FindBtn.Sensitive = true;
			else
				FindBtn.Sensitive = false;
		}
		
		
		/* SearchKeyword activate */
		protected void on_SearchKeyword_activate (object o, EventArgs args)
		{
			if (SearchKeyword.Text != "")
				on_FindBtn_clicked (null, null);
		}
		
		
		/* FindButton clicked */
		protected void on_FindBtn_clicked (object o, EventArgs args)
		{
			SearchResultsBg.Show ();
			ResetBrowsingSettings (ShowEntries.Active == (int) ShowEntriesEnum.ImportedEntries);
			ReloadEntries ();
		}
		
		
		/* SearchKeyword key presed */
		protected void on_SearchKeyword_key_press_event (object o, KeyPressEventArgs args)
		{
			if (args.Event.Key == Gdk.Key.Escape)
				FindTggBtn.Active = false;
		}
		
		
		/* CloseImportEntries button clicked */
		protected void on_CloseImportedEntries_clicked (object o, EventArgs args)
		{
			RestoreBrowsingSettings ();
		}
		
		
		/* CloseSearchResults button clicked */
		protected void on_CloseSearchResults_clicked (object o, EventArgs args)
		{
			FindTggBtn.Active = false;
		}
		
		
		protected void on_HideSidebar_clicked (object o, EventArgs args)
		{
			show_tags_sidebar.Active = ! show_tags_sidebar.Active;
		}
		
		
		protected void on_ZoomIn_clicked (object o, EventArgs args)
		{
			SetSensitive (false);
			
			int zoom = Engine.Settings.GetInt ("zoom");
			if (zoom == -1)
				zoom = 100;
			
			zoom += 20;
			Entries.Zoom (zoom);
			
			Engine.Settings.SetInt ("zoom", zoom);
			ZoomPercent.Text = zoom.ToString () + "%";
			
			SetSensitive (true);
		}
		
		
		protected void on_ZoomOut_clicked (object o, EventArgs args)
		{
			SetSensitive (false);
			
			int zoom = Engine.Settings.GetInt ("zoom");
			if (zoom == -1)
				zoom = 100;
			
			zoom -= 20;
			Entries.Zoom (zoom);
			
			Engine.Settings.SetInt ("zoom", zoom);
			ZoomPercent.Text = zoom.ToString () + "%";
			
			SetSensitive (true);
		}
		


		////////////////////////////////////////////////////////////////////////
		//
		//   window functions
		//
		////////////////////////////////////////////////////////////////////////
		
		
		/// <summary>Sets Main window sensitive</summary>
		public void SetSensitive (bool sensitive)
		{
			window.Sensitive = sensitive;
			while (Application.EventsPending ())
				Application.RunIteration ();
		}
		
		
		/// <summary>Shows window message</summary>
		public void ShowWindowMessage (string title, string text)
		{
			MessageA.Markup = "<span size='xx-large' weight='bold'>" + title + "</span>";
			MessageB.Markup = text;
			
			Pages.Page = (int) PagesEnum.Message;
			Tools.Repaint ();
		}
		
		
		/// <summary>Hides window message</summary>
		public void HideWindowMessage ()
		{
			Pages.Page = (int) PagesEnum.Main;
		}
		
		
		/// <summary>Shows entries message.</summary>
		public void ShowEntriesMessage (string title, string text)
		{
			EntriesMsg.Markup = "<span size='xx-large' weight='bold'>" + title + "</span>\n\n" + text;
			EntriesListNotebook.Page = (int) EntriesListNotebookEnum.Message;
			Tools.Repaint ();
		}
		
		
		/// <summary>Hides entries message.</summary>
		public void HideEntriesMessage ()
		{
			EntriesListNotebook.Page = (int) EntriesListNotebookEnum.Normal;
		}
		
		
		/* Saves browsing settings and resets them. */
		private void ResetBrowsingSettings (bool ImportedEntries)
		{
			NoReloadEntries = true;
			
			BrowsingSettings.show    = ShowEntries.Active;
			BrowsingSettings.tags    = Tags.DisabledTagsList ();
			BrowsingSettings.restore = true;
			
			if (ImportedEntries)
				ShowEntries.Active = (int) ShowEntriesEnum.ImportedEntries;
			else
				ShowEntries.Active = (int) ShowEntriesEnum.AllEntries;
			
			ShowAllTags ();
			
			NoReloadEntries = false;
			ReloadEntries ();
		}
		
		
		/* Restores browsing settings */
		private void RestoreBrowsingSettings ()
		{
			ImportedEntriesBg.Visible = false;
			
			if (BrowsingSettings.restore == false)
				return;
			
			NoReloadEntries = true;
			
			Tags.LoadTagsActive (new ArrayList (BrowsingSettings.tags));
			
			ShowEntries.Active = BrowsingSettings.show;
			BrowsingSettings.restore = false;
			
			NoReloadEntries = false;
			ReloadEntries ();
		}
		
		
		public void NoEntriesMessage ()
		{
			if (Entries.NEntries > 0)
				HideEntriesMessage ();
			else
				if (DB.NEntries > 0) {
					if (SearchResultsBg.Visible == true) // if showing search results
						ShowEntriesMessage (Catalog.GetString ("No results were found !"), "");
					else
						ShowEntriesMessage (Catalog.GetString ("Entries couldn't be showed !"),
											Catalog.GetString ("Change browsing settings to see entries."));
				} else
					ShowEntriesMessage (Catalog.GetString ("There are no entries !"),
										Catalog.GetString ("To add a new entry click <u>Add entry</u> button."));
		}
		
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   TrayIcon
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void TrayIcon_on_Exit_activated (object o, EventArgs args)
		{
			ExitApplication ();
		}
		
		
		protected void TrayIcon_on_AddEntry_activated (object o, EventArgs args)
		{
			on_add_activate (null, null);
		}
		
		
		protected void TrayIcon_on_DragData_received (object o, DragDataReceivedArgs args)
		{
			string url = new System.Text.StringBuilder (System.Text.Encoding.UTF8.GetString (args.SelectionData.Data)).ToString ();
			
			try {
				Uri addr = new Uri (url);
					
				url = url.Replace ("file:///", "").Trim ();
				
				if (! Engine.FilesPaths.IsWinSystem ())
					url = "/" + url;
				else {
					url = url.Replace (Convert.ToChar (13), '*');
					url = url.Replace (Convert.ToChar (10), '*');
					url = url.Replace ('/', '\\');
					url = url.Replace ("*", "");
				}
			
				if ((addr.IsFile == true) && (System.IO.File.Exists (url.ToString ()))) { // if file
					// checking file extension
					if (System.IO.Path.GetExtension (url) != ".ied") {
						window.Sensitive = false;
						MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Error,
								ButtonsType.Ok, Catalog.GetString ("The file has an invalid extension."));
						msg.Run ();
						msg.Destroy ();
						window.Sensitive = true;
						return;
					}
					
					on_import_entries_activate (url, null);
				} else
					if ((addr.ToString ().StartsWith ("http")) || (addr.ToString ().StartsWith ("ftp")))
						AddEntry ("webaddress", addr.ToString ());	
			} catch {
				return;
			}
		}
	}
}
