/*
 *  GUI/PluginsDialog.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Gdk;
using Mono.Unix;
using Gtk;
using Glade;
using Incollector.Plugins;
using System.IO;


namespace Incollector.GUI {
	
	/*
	 * PluginsDialog class
	 */
	
	public class PluginsDialog {
		private Glade.XML  gui;
		public  Gtk.Dialog dialog;
		
		[Glade.Widget] Gtk.TreeView PluginsList;
		[Glade.Widget] Gtk.Button   RemoveBtn, AboutBtn;
		[Glade.Widget] Gtk.HButtonBox HButtonBox1;
		[Glade.Widget] Gtk.Label    MsgLabel, ResMsgLbl, IncollectorLabel;
		[Glade.Widget] Gtk.Frame    IncollectorFrame, PluginsFrame, LibrariesFrame;
		[Glade.Widget] Gtk.VBox     LibrariesVBox, PluginsVBox;
		[Glade.Widget] Gtk.Viewport MsgBg;
		[Glade.Widget] Gtk.Notebook MainNotebook;
		
		// from AboutDialog
		[Glade.Widget] Gtk.Label PluginName, Version, Description, Url, Copyright, Authors;
		[Glade.Widget] Gtk.Image PluginIcon;
		
		
		enum ColsEnum {
			Name    = 0,
			Version = 1,
			ID      = 2
		}
		
		
		/* Constructor */
		public PluginsDialog (Gtk.Window parent)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "PluginsDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("PluginsDialog");
			dialog.Sensitive = false;
			dialog.VBox.BorderWidth = 0;
			dialog.TransientFor = parent;

			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.PLUGINS_DIALOG, "Icon", gui);
			Icons.AssignImage (Icons.ADD_PLUGIN_BUTTON, "AddBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_PLUGIN_BUTTON, "RemoveBtnImg", gui);
			
			Colors.ColorHead (Engine.Settings.GetBool ("ColorWindowsHeaders"), gui);
			Colors.Modify (Colors.WHITE, "ListViewport", gui);
			Colors.Modify (Colors.WARNING_MSG, "MsgBg", gui);
			
			
			PluginsList.Selection.Changed += new EventHandler (on_PluginsList_selection_changed);
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (string), typeof (int));
			PluginsList.Model = store;
			
			CellRendererText txt = new CellRendererText ();
			txt.Ellipsize = Pango.EllipsizeMode.End;
			TreeViewColumn col = new TreeViewColumn (Catalog.GetString ("Plugin"), txt, "markup", (int) ColsEnum.Name);
			col.Expand = true;
			PluginsList.AppendColumn (col);
			
			PluginsList.AppendColumn (Catalog.GetString ("Version"), new CellRendererText (), "text", (int) ColsEnum.Version);
			
			
			int  i = 0;
			foreach (Plugin plug in Incollector.Plugins.Plugins.PluginsList) {
				store.AppendValues (new object [] {"<b>" + plug.Title + "</b>\n<span size='small'>" + plug.Description + "</span>", plug.Version, i});
				i++;
			}
			
			
			dialog.Sensitive = true;
		}
		
		protected void on_PluginsList_selection_changed (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			bool sens = AboutBtn.Sensitive = PluginsList.Selection.GetSelected (out mod, out itr);
			if (sens) {
				Plugin plug = Plugins.Plugins.PluginsList [(int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID)] as Plugin;
				System.IO.FileInfo inf = new System.IO.FileInfo (plug.Path);
				
				if (inf.FullName.StartsWith (Engine.FilesPaths.UserPluginsDirectory) == false)
					sens = false;
			}
			
			RemoveBtn.Sensitive = sens;
		}
		
		
		protected void on_PluginsList_row_activated (object o, RowActivatedArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (PluginsList.Selection.GetSelected (out mod, out itr)) {
				dialog.Sensitive = false;
				
				Glade.XML DlgXml = Glade.XML.FromAssembly ("gui.glade", "AboutPluginDialog", null);
				DlgXml.Autoconnect (this);
				Gtk.Dialog dlg = (Gtk.Dialog) DlgXml.GetWidget ("AboutPluginDialog");
				dlg.VBox.BorderWidth = 0;
				dlg.Modal = true;
				dlg.TransientFor = dialog;
				Icons.AssignWindow (dlg);
				Colors.Modify (Colors.HEAD, "Bg", DlgXml);
				Colors.Modify (Colors.HEAD, "Bg2", DlgXml);
				
				int id = (int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID);
				PluginName.Markup  = "<span size='x-large' weight='bold'>" +
							(Plugins.Plugins.PluginsList [id] as Plugin).Title + "</span>";
				
				PluginIcon.Pixbuf  = (Plugins.Plugins.PluginsList [id] as Plugin).Icon;
				Version.Markup     = (Plugins.Plugins.PluginsList [id] as Plugin).Version;
				Description.Markup = (Plugins.Plugins.PluginsList [id] as Plugin).Description;
				Url.Markup         = (Plugins.Plugins.PluginsList [id] as Plugin).Url;
				Copyright.Markup   = (Plugins.Plugins.PluginsList [id] as Plugin).Copyright;
				Authors.Markup     = (Plugins.Plugins.PluginsList [id] as Plugin).Authors;

				dlg.VBox.BorderWidth = 0;
				dlg.Run ();
				dlg.Destroy ();
				
				dialog.Sensitive = true;
			}
		}
		
		
		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			dialog.Sensitive = false;
			
			AddingPluginDialog dlg = new AddingPluginDialog (dialog, "");
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				HButtonBox1.Hide ();
				MainNotebook.Page = 1;
				Tools.Repaint ();
				
				ArrayList Dependences;
				
				switch (Plugins.Plugins.InstallPlugin (dlg.FileName.Filename, out Dependences)) {
					case InstallationResult.AlreadyExists:
						ResMsgLbl.Markup = "<span size='x-large' weight='bold'>" + 
										Catalog.GetString ("Plugin hasn't been installed !") +
										"</span>\n\n<span size='large'>" +
										Catalog.GetString ("This plugin is already installed."
										+ "</span>");
					break;
					
					case InstallationResult.OlderVersion:
						ResMsgLbl.Markup = "<span size='x-large' weight='bold'>" + 
										Catalog.GetString ("Plugin hasn't been installed !") +
										"</span>\n\n<span size='large'>" +
										Catalog.GetString ("Newer version already exists."
										+ "</span>");
					break;
					
					case InstallationResult.PluginUpdated:
						ResMsgLbl.Markup = "<span size='x-large' weight='bold'>" + 
										Catalog.GetString ("Plugin has been installed !") +
										"</span>\n\n<span size='large'>" +
										Catalog.GetString ("Older version has been updated."
										+ "</span>");
						ShowChangesMessage ();
					break;
					
					case InstallationResult.Successful:
						ResMsgLbl.Markup = "<span size='x-large' weight='bold'>" + 
										Catalog.GetString ("Plugin has been installed !") +
										"</span>\n\n<span size='large'>" +
										Catalog.GetString ("Your new plugin has been installed successfully !"
										+ "</span>");
						ShowChangesMessage ();
					break;
					
					case InstallationResult.Dependences:
						MainNotebook.Page = 2;
						
						foreach (PluginInfo.Dep dep in Dependences) {
							if (dep.type == PluginInfo.DepType.Incollector) {
								IncollectorFrame.Show ();
								IncollectorLabel.Markup = "<b>Incollector " + dep.version.ToString () + "</b>";
							} else if (dep.type == PluginInfo.DepType.Library) {
								Gtk.Label label = new Gtk.Label (dep.name + "   " + dep.version.ToString ());
								label.Xalign = (float) 0.00;
								LibrariesVBox.Add (label);
								LibrariesFrame.ShowAll ();
							} else { // plugin
								Gtk.Label label = new Gtk.Label (dep.name + "   " + dep.version.ToString ());
								label.Xalign = (float) 0.00;
								PluginsVBox.Add (label);
								PluginsFrame.ShowAll ();
							}
						}
					break;
				}
			}
			
			dlg.dialog.Destroy ();
			dialog.Sensitive = true;
		}
		
		
		protected void on_ContinueBtn_clicked (object o, EventArgs args)
		{
			HButtonBox1.Show ();
			MainNotebook.Page = 0;
		}
		
		
		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (PluginsList.Selection.GetSelected (out mod, out itr)) {
				Plugin plg = (Plugin) Plugins.Plugins.PluginsList [(int) (mod as Gtk.ListStore).GetValue (itr, (int) ColsEnum.ID)];
				(mod as ListStore).Remove (ref itr);
				
				// Adding plugin to PluginsToRemove list. It will be removed on application starting. 
				ArrayList list = Engine.Settings.GetList ("PluginsToRemove");
				list.Remove (plg.Path);
				list.Add (plg.Path);
				Engine.Settings.SetList ("PluginsToRemove", list);
				
				ShowChangesMessage ();
			}
		}
		
		
		protected void on_AboutBtn_clicked (object o, EventArgs args)
		{
			on_PluginsList_row_activated (null, null);
		}
		
		
		protected void ShowChangesMessage ()
		{
			MsgLabel.Text = Catalog.GetString ("All changes will take effect after restarting Incollector.");
			MsgBg.Show ();
		}
	}
}
