/*
 *  LinkLabel.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;
using Gtk;
using Mono.Unix;


namespace Incollector.GUI.Widgets {
	public class LinkLabel : Gtk.HBox {
		Gtk.Label label;
		string    Url;
		
		
		public LinkLabel (string Text, string Url)
		{
			CreateLinkLabel (Text, Url);
		}
		
		
		public LinkLabel (string Url)
		{
			CreateLinkLabel (Url, Url);
		}
		
		
		public void CreateLinkLabel (string Text, string Url)
		{
			this.Url = Url;
			
			Gtk.EventBox evbox = new Gtk.EventBox ();
			evbox.VisibleWindow = false;
			evbox.ButtonPressEvent += new ButtonPressEventHandler (on_Clicked);
			evbox.EnterNotifyEvent += new EnterNotifyEventHandler (on_Enter);
			evbox.LeaveNotifyEvent += new LeaveNotifyEventHandler (on_Leave);
			
			label = new Label (Url);
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (16, 16, 239));
			label.Text   = Text;
			label.Xalign = 0;
			label.Yalign = (float) 0.50;
			evbox.Add (label);
			
			this.Add (evbox);
			
			
			this.ShowAll ();
		}
		
		
		protected void on_Enter (object o, EnterNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (96, 128, 255));
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
		}
		
		
		protected void on_Leave (object o, LeaveNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (16, 16, 239));
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.TopLeftArrow);
		}
		
		
		protected void on_Clicked (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 1)
				Tools.RunWebBrowser (Url);
			else if (args.Event.Button == 3) {
				Menu menu = new Menu ();
				
				Gtk.ImageMenuItem OpenLink = new Gtk.ImageMenuItem (Catalog.GetString ("Open link"));
				OpenLink.Image = new Gtk.Image (Stock.JumpTo, Gtk.IconSize.Menu);
				OpenLink.Activated += new EventHandler (on_OpenLink_activate);
				menu.Add (OpenLink);
				
				Gtk.ImageMenuItem CopyLink = new Gtk.ImageMenuItem (Catalog.GetString ("Copy link address"));
				CopyLink.Image = new Gtk.Image (Stock.Copy, Gtk.IconSize.Menu);
				CopyLink.Activated += new EventHandler (on_CopyLink_activate);
				menu.Add (CopyLink);
				
				menu.ShowAll ();
				menu.Popup ();
			}
		}
		
		
		protected void on_OpenLink_activate (object o, EventArgs args)
		{
			Tools.RunWebBrowser (label.Text);
		}
		
		
		protected void on_CopyLink_activate (object o, EventArgs args)
		{
			Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", true)).Text = Url;
		}
	}
}
