/*
 *  TagsEntry.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Collections;
using GLib;
using Gtk;
using Mono.Unix;


namespace Incollector.GUI.Widgets {
	public class TagsEntry : Gtk.Entry {
		
		Gtk.Window   WordsWindow;
		Gtk.TreeView WordsList;
		string []    Tags;
		
		
		public TagsEntry (string [] Tags)
		{
			this.Tags = Tags;
			
			// creating words window
			WordsWindow = new Gtk.Window ("");
			WordsWindow.Decorated = false;
			WordsWindow.TypeHint = Gdk.WindowTypeHint.Menu;
			WordsWindow.FocusOutEvent += new Gtk.FocusOutEventHandler (on_WordsWindow_focus_out);
			
			
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic);
			scroll.ShadowType = Gtk.ShadowType.In;
			scroll.WidthRequest = 200;
			scroll.HeightRequest = 150;
			WordsWindow.Add (scroll);
			
			WordsList = new Gtk.TreeView ();
			WordsList.Model = new Gtk.ListStore (typeof (string));
			WordsList.EnableSearch = false;
			WordsList.RulesHint = true;
			WordsList.HeadersVisible = false;
			WordsList.PangoContext.FontDescription.Size = (int) (Style.FontDescription.Size * 90 / 100);
			WordsList.AppendColumn ("", new CellRendererText (), "text", 0);
			scroll.Add (WordsList);
			
			scroll.ShowAll ();
			
			
			// creating entry
			this.KeyReleaseEvent += new Gtk.KeyReleaseEventHandler (on_key_release);
			this.KeyPressEvent += new Gtk.KeyPressEventHandler (on_key_press);
			this.ShowAll ();
		}
		
		
		protected void on_WordsWindow_focus_out (object obj, Gtk.FocusOutEventArgs args)
		{
			WordsWindow.Modal = false;
			WordsWindow.Hide ();
		}
		
		
		protected void on_key_press (object o, KeyPressEventArgs args)
		{
			switch (args.Event.Key) {
				case Gdk.Key.Escape:
					WordsWindow.Hide ();
					args.RetVal = true;
				break;
				
				case Gdk.Key.Up:
					WordsList.ProcessEvent (args.Event);
					
					Gtk.TreeIter  itr2;
					Gtk.TreeModel mod2;
					
					if (WordsList.Selection.GetSelected (out mod2, out itr2)) {
						Gtk.TreePath pth1 = (mod2 as ListStore).GetPath (itr2);
						pth1.Prev ();
						WordsList.Selection.SelectPath (pth1);
					}
					
					args.RetVal = true;
				break;
				
				case Gdk.Key.Down:
					Gtk.TreeIter  itr3;
					Gtk.TreeModel mod3;
					
					if (WordsList.Selection.GetSelected (out mod3, out itr3)) {
						Gtk.TreePath pth2 = (mod3 as ListStore).GetPath (itr3);
						pth2.Next ();
						WordsList.Selection.SelectPath (pth2);
					}
					
					args.RetVal = true;
				break;
			}
		}
		
		
		protected void on_key_release (object o, KeyReleaseEventArgs args)
		{
			switch (args.Event.Key) {
				case Gdk.Key.Return:
					if (WordsWindow.Visible == false)
						return;
					
					Gtk.TreeIter  itr3;
					Gtk.TreeModel mod3;
					
					if (WordsList.Selection.GetSelected (out mod3, out itr3))
						FinishCurrentWord ((string) (mod3 as ListStore).GetValue (itr3, 0));
				break;
				
				case Gdk.Key.Escape: break;
				case Gdk.Key.Up: break;
				case Gdk.Key.Down: break;
				
				default:
					ShowWords (Search (GetCurrentWord ()));
				break;
			}
		}
		
		
		protected ArrayList Search (string StrBegin)
		{
			if (StrBegin == "")
				return new ArrayList ();
			
			ArrayList list = new ArrayList ();
			
			foreach (string tag in Tags)
				if ((tag.ToUpper ().StartsWith (StrBegin.ToUpper ())) && (string.Compare (tag.ToUpper (), StrBegin.ToUpper ()) != 0))
					list.Add (tag);
			
			list.Sort ();
			
			return list;
		}
		
		
		protected string GetCurrentWord ()
		{
			string word = "";
			
			for (int i = this.Position - 1; (i >= 0) && (this.Text [i] != ','); i--)
				word = this.Text [i] + word;
			
			return word.Trim ();
		}
		
		
		protected void FinishCurrentWord (string NewWord)
		{
			int pos = this.Position;
			for (int i = this.Position - 1; (i >= 0) && (this.Text [i] != ','); i--)
				if (this.Text [i] != ' ')
					pos = i;
			
			this.DeleteText (pos, this.Position);
			this.InsertText (NewWord, ref pos);
			this.Position = pos;
			
			WordsWindow.Hide ();
		}
		
		
		protected void ShowWords (ArrayList Words)
		{
			if (Words.Count == 0) {
				WordsWindow.Hide ();
				return;
			}
			
			(WordsList.Model as ListStore).Clear ();
			foreach (string word in Words)
				(WordsList.Model as ListStore).AppendValues (new object [] { word });
			
			Gtk.TreeIter FirstIter;
			(WordsList.Model as ListStore).GetIterFirst (out FirstIter);
			WordsList.Selection.SelectPath ((WordsList.Model as ListStore).GetPath (FirstIter));
			
			
			int x, y;
			this.GdkWindow.GetOrigin (out x, out y);
			
			int width, height;
			this.GdkWindow.GetSize (out width, out height);
			
			WordsWindow.Move (x, y + height);
			WordsWindow.Show ();
		}
	}
}
