/*
 *  Plugins/PluginInfo.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using System.Reflection;
using System.IO;
using System.Xml;
using Gtk;


namespace Incollector.Plugins {

	public class PluginInfo {
		
		protected Version   _Version;
		protected ArrayList Dependences;
		
		
		/// <summary>Dependence type</summary>
		public enum DepType {
			Incollector = 0,
			Library     = 1,
			Plugin      = 2
		};
		
		
		public struct Dep {
			public string  name;
			public Version version;
			public DepType type;
			
			public Dep (string Ver, string Name, DepType type)
			{
				this.name    = Name;
				this.version = new System.Version (Ver);
				this.type    = type;
			}
		}
		
		
		public Version version {
			get { return _Version; }
		}
		
	
		public PluginInfo (string FileName)
		{
			_Version = new Version (0, 0, 0, 0);
			Dependences = new ArrayList ();
			
			StreamReader reader;
			try {
				reader = new StreamReader (FileName, System.Text.Encoding.UTF8);
			} catch  {
				return;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						try {
							if ((read.Name == "version") && (read.NodeType == XmlNodeType.Element))
								this._Version = new Version (read.ReadString ());
						} catch {}
						
						try {
							if ((read.Name == "incollector") && (read.NodeType == XmlNodeType.Element))
								Dependences.Add (new Dep (read.GetAttribute ("version"),
												Assembly.GetCallingAssembly ().GetName ().Name,
												DepType.Incollector));
						} catch {}
						
						try {
							if ((read.Name == "library") && (read.NodeType == XmlNodeType.Element))
								Dependences.Add (new Dep (read.GetAttribute ("version"),
												read.ReadString (), DepType.Library));
						} catch {}
						
						try {
							if ((read.Name == "plugin") && (read.NodeType == XmlNodeType.Element))
								Dependences.Add (new Dep (read.GetAttribute ("version"),
												read.ReadString (), DepType.Plugin));
						} catch {}
					}
				}
			} catch {
				reader.Close ();
				return;
			}
			

			reader.Close ();
		}
		
		
		public ArrayList CheckDependences ()
		{
			ArrayList rtn = new ArrayList ();
			
			foreach (Dep dep in Dependences) {
				foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies ()) {
					AssemblyName name = asm.GetName ();
					if (name.Name == dep.name) {
						if (name.Version < dep.version)
							rtn.Add (dep);
						
						goto next;
					}
				}
				
				rtn.Add (dep);
				next: ;
			}
			
			return rtn;
		}
	}
}
