/*
 *  Types/Conversation.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {
	
	public class Conversation : DataType {
		private struct ConversationType {
			public string title;
			public string text;
		}
		
		
		private Gtk.Entry    EditTitle;
		private Gtk.TextView EditText;
		
		
		public override string Name {
			get { return "conversation"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Conversation"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a conversation log"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditTitle; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditTitle; }
		}
		
		
		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			ConversationType cnv = new ConversationType ();
			cnv.title = reader.GetAttribute ("title");
			cnv.text  = reader.ReadString ();
			entry.type = Name;
			entry.data = cnv;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":title", cnv.title);
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			ConversationType cnv = (ConversationType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("title", cnv.title);
				writer.WriteString (cnv.text);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Title"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditTitle = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditTitle, "data:" + Name + ":title");
			tbl.Attach (EditTitle, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Conversation"));
			lbl2.Xalign = 1;
			lbl2.Yalign = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditText = new Gtk.TextView ();
			EditText.HeightRequest = 150;
			EditText.AcceptsTab = false;
			EditText.WrapMode = Gtk.WrapMode.Word;
			EditText.PixelsAboveLines = 1;
			EditText.LeftMargin = 3;
			EditText.RightMargin = 3;
			
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.In; 
			scroll.Add (EditText);
			tbl.Attach (scroll, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditTitle.Text = "";
				EditText.Buffer.Text = content;
			} else {
				ConversationType cnv = (ConversationType) entry.data;
				EditTitle.Text = cnv.title;
				EditText.Buffer.Text = cnv.text;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditTitle != null) && (EditText != null)) {
				EditTitle.Changed        += hdlr;
				EditText.Buffer.Changed  += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			ConversationType cnv = (ConversationType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Title") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label LabelTitle  = new Gtk.Label (cnv.title);
			LabelTitle.Xalign     = 0;
			LabelTitle.Yalign     = 0;
			LabelTitle.Selectable = true;
			
			tbl.Attach (LabelTitle, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Conversation") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3); 
			
			Gtk.Label LabelConversation = new Gtk.Label (cnv.text);
			LabelConversation.Wrap = true;
			LabelConversation.Xalign = 0;
			LabelConversation.Yalign = 0;
			LabelConversation.Selectable = true;
			tbl.Attach (LabelConversation, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			ConversationType cnv = (ConversationType) entry.data;
			return cnv.title;
		}
		
		
		public override object GetData ()
		{
			ConversationType cnv = new ConversationType ();
			cnv.title = EditTitle.Text;
			cnv.text  = EditText.Buffer.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":title", cnv.title);
			
			return cnv;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			ConversationType cnv = (ConversationType) entry.data;
			return Engine.Text.Search (query, cnv.title) || Engine.Text.Search (query, cnv.text);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditTitle != null) && (EditText != null) &&
					(EditTitle.Text != "") && (EditText.Buffer.Text != "");
		}
	}
}
