/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {
	
	public class Note : DataType {
		private struct NoteType {
			public string title;
			public string text;
		}
	
	
		private Gtk.Entry    EditTitle;
		private Gtk.TextView EditNote;
		
		
		public override string Name {
			get { return "note"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Note"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a note"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditTitle; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditNote; }
		}
		
		
		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			NoteType nte = new NoteType ();
			
			if (reader.AttributeCount > 0)
				nte.title = reader.GetAttribute ("title");
			else
				nte.title = "";
			nte.text  = reader.ReadString ();
			
			entry.type = Name;
			entry.data = nte;
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			NoteType nte = (NoteType) entry.data;
			
			writer.WriteStartElement (Name);
			writer.WriteAttributeString ("title", nte.title);
				writer.WriteString (nte.text);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Title"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditTitle = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditTitle, "data:" + Name + ":title");
			tbl.Attach (EditTitle, 1, 2, 0, 1,
						AttachOptions.Shrink | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Note"));
			lbl2.Xalign = 1;
			lbl2.Yalign = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditNote = new Gtk.TextView ();
			EditNote.HeightRequest = 80;
			EditNote.AcceptsTab = false;
			EditNote.WrapMode = Gtk.WrapMode.Word;
			EditNote.PixelsAboveLines = 1;
			EditNote.LeftMargin = 3;
			EditNote.RightMargin = 3;
			
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.In; 
			scroll.Add (EditNote);
			tbl.Attach (scroll, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditTitle.Text = "";
				EditNote.Buffer.Text = content;
			} else {
				NoteType nte = (NoteType) entry.data;
				EditTitle.Text = nte.title;
				EditNote.Buffer.Text = nte.text;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if (EditNote != null)
				EditNote.Buffer.Changed += hdlr;
			
			if (EditTitle != null)
				EditTitle.Changed += hdlr;
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			NoteType nte = (NoteType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Title") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label LabelAuthor  = new Gtk.Label (nte.title);
			LabelAuthor.Xalign     = 0;
			LabelAuthor.Yalign     = 0;
			LabelAuthor.Selectable = true;
			
			tbl.Attach (LabelAuthor, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Note") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label LabelQuote  = new Gtk.Label (nte.text);
			LabelQuote.Wrap       = true;
			LabelQuote.Xalign     = 0;
			LabelQuote.Yalign     = 0;
			LabelQuote.Selectable = true;
			tbl.Attach (LabelQuote, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			NoteType nte = (NoteType) entry.data;
			
			if (nte.title != "")
				return nte.title;
			else
				return nte.text;
		}
		
		
		public override object GetData ()
		{
			NoteType nte = new NoteType ();;
			nte.title = EditTitle.Text;
			nte.text  = EditNote.Buffer.Text;
			
			return nte;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			NoteType nte = (NoteType) entry.data;
			return (Engine.Text.Search (query, nte.title)) || (Engine.Text.Search (query, nte.text));
		}
		
		
		public override bool IsFormFilledIn ()
		{
			if ((EditTitle != null) && (EditNote != null))
				return (EditTitle.Text != "") && (EditNote.Buffer.Text != "");
			else
				return false;
		}
	}
}
