/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class Quote : DataType {
		private struct QuoteType {
			public string author;
			public string quote;
		}
		
		
		private Gtk.Entry    EditAuthor;
		private Gtk.TextView EditQuote;
		
		
		public override string Name {
			get { return "quote"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Quote"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditAuthor; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditAuthor; }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a quote"); }
		}


		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			QuoteType qte = new QuoteType ();
			qte.author = reader.GetAttribute ("author");
			qte.quote  = reader.ReadString ();
			entry.type = Name;
			entry.data = qte;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":author", qte.author);
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			QuoteType qte = (QuoteType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("author", qte.author);
				writer.WriteString (qte.quote);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Author"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditAuthor = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditAuthor, "data:" + Name + ":author");
			tbl.Attach (EditAuthor, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Quote"));
			lbl2.Xalign = 1;
			lbl2.Yalign = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditQuote = new Gtk.TextView ();
			EditQuote.HeightRequest = 80;
			EditQuote.AcceptsTab = false;
			EditQuote.WrapMode = Gtk.WrapMode.Word;
			EditQuote.PixelsAboveLines = 1;
			EditQuote.LeftMargin = 3;
			EditQuote.RightMargin = 3;
			
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.In; 
			scroll.Add (EditQuote);
			tbl.Attach (scroll, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditAuthor.Text = "";
				EditQuote.Buffer.Text = content;
			} else {
				QuoteType qte = (QuoteType) entry.data;
				EditAuthor.Text = qte.author;
				EditQuote.Buffer.Text = qte.quote;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditAuthor != null) && (EditQuote != null)) {
				EditAuthor.Changed       += hdlr;
				EditQuote.Buffer.Changed += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			QuoteType qte = (QuoteType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Author") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label LabelAuthor  = new Gtk.Label (qte.author);
			LabelAuthor.Xalign     = 0;
			LabelAuthor.Yalign     = 0;
			LabelAuthor.Selectable = true;
			
			tbl.Attach (LabelAuthor, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Quote") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label LabelQuote  = new Gtk.Label (qte.quote);
			LabelQuote.Wrap       = true;
			LabelQuote.Xalign     = 0;
			LabelQuote.Yalign     = 0;
			LabelQuote.Selectable = true;
			tbl.Attach (LabelQuote, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			QuoteType qte = (QuoteType) entry.data;
			return qte.quote;
		}
		
		
		public override object GetData ()
		{
			QuoteType qte = new QuoteType ();
			qte.author = EditAuthor.Text;
			qte.quote  = EditQuote.Buffer.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":author", qte.author);
			
			return qte;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			QuoteType qte = (QuoteType) entry.data;
			return Engine.Text.Search (query, qte.author) || Engine.Text.Search (query, qte.quote);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditAuthor != null) && (EditQuote != null) &&
				(EditAuthor.Text != "") && (EditQuote.Buffer.Text != "");
		}
	}
}
