/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {
	
	public class SerialNumber : DataType {
		private struct SerialNumberType {
			public string application;
			public string number;
		}


		private Gtk.Entry EditApplication;
		private Gtk.Entry EditNumber;
	
	
		public override string Name {
			get { return "serial"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Serial number"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a serial number"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditApplication; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditApplication; }
		}
		

		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			SerialNumberType srl = new SerialNumberType ();
			srl.application = reader.GetAttribute ("application");
			srl.number      = reader.ReadString ();
			entry.type = Name;
			entry.data = srl;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":application", srl.application);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":number", srl.number);
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			SerialNumberType srl = (SerialNumberType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("application", srl.application);
				writer.WriteString (srl.number);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Application"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditApplication = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditApplication, "data:" +
															Name + ":application");
			tbl.Attach (EditApplication, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Number"));
			lbl2.Xalign = 1;
			lbl2.Yalign = (float) 0.50;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditNumber = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditNumber, "data:" + Name + ":number");
			tbl.Attach (EditNumber, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditApplication.Text = "";
				EditNumber.Text = content;
			} else {
				SerialNumberType srl = (SerialNumberType) entry.data;
				EditApplication.Text = srl.application;
				EditNumber.Text = srl.number;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditApplication != null) && (EditNumber != null)) {
				EditApplication.Changed += hdlr;
				EditNumber.Changed      += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			SerialNumberType srl = (SerialNumberType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Application") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Application  = new Gtk.Label (srl.application);
			Application.Xalign     = 0;
			Application.Yalign     = 0;
			Application.Selectable = true;
			tbl.Attach (Application, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Serial number") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label SerialNumber = new Gtk.Label (srl.number);
			SerialNumber.Wrap = true;
			SerialNumber.Xalign = 0;
			SerialNumber.Yalign = 0;
			SerialNumber.Selectable = true;
			tbl.Attach (SerialNumber, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			SerialNumberType srl = (SerialNumberType) entry.data;
			return srl.application;
		}
		
		
		public override object GetData ()
		{
			SerialNumberType srl = new SerialNumberType ();
			srl.application = EditApplication.Text;
			srl.number      = EditNumber.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":application", srl.application);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":number", srl.number);
			
			return srl;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			SerialNumberType srl = (SerialNumberType) entry.data;
			return Engine.Text.Search (query, srl.application) || Engine.Text.Search (query, srl.number);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditApplication != null) && (EditNumber != null) &&
				(EditApplication.Text != "") && (EditNumber.Text != "");
		}
	}
}
