/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class SourceCode : DataType {
		private struct SourceCodeType {
			public string language;
			public string code;
		}
		
		
		private Gtk.Entry    EditLanguage;
		private Gtk.TextView EditCode;
		
		
		public override string Name {
			get { return "sourcecode"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Source code"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a source code"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditLanguage; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditLanguage; }
		}


		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			SourceCodeType src = new SourceCodeType ();
			src.language = reader.GetAttribute ("language");
			src.code     = reader.ReadString ();
			entry.type = Name;
			entry.data = src;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", src.language);			
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			SourceCodeType src = (SourceCodeType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("language", src.language);
				writer.WriteString (src.code);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Language"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditLanguage = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditLanguage, "data:" +
														Name + ":language");
			tbl.Attach (EditLanguage, 1, 2, 0, 1,
						AttachOptions.Shrink | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Code"));
			lbl2.Xalign = 1;
			lbl2.Yalign = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditCode = new Gtk.TextView ();
			EditCode.HeightRequest = 150;
			EditCode.AcceptsTab = false;
			EditCode.WrapMode = Gtk.WrapMode.Word;
			EditCode.PixelsAboveLines = 1;
			EditCode.LeftMargin = 3;
			EditCode.RightMargin = 3;
			
			Pango.FontDescription fnt = new Pango.FontDescription ();
			if (Engine.FilesPaths.IsWinSystem () == true)
				fnt.Family = "Courier New";
			else
				fnt.Family = "Monospace";
			
			EditCode.ModifyFont (fnt);
			
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.In; 
			scroll.Add (EditCode);
			tbl.Attach (scroll, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditLanguage.Text = "";
				EditCode.Buffer.Text = content;
			} else {
				SourceCodeType cnv = (SourceCodeType) entry.data;
				EditLanguage.Text = cnv.language;
				EditCode.Buffer.Text = cnv.code;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditLanguage != null) && (EditCode != null)) {
				EditLanguage.Changed += hdlr;
				EditCode.Buffer.Changed += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			SourceCodeType src = (SourceCodeType) entry.data;
			
			Pango.FontDescription fnt = new Pango.FontDescription ();
			if (Engine.FilesPaths.IsWinSystem () == true)
				fnt.Family = "Courier New";
			else
				fnt.Family = "Monospace";
			
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Language") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Language  = new Gtk.Label (src.language);
			Language.Xalign     = 0;
			Language.Yalign     = 0;
			Language.Selectable = true;
			tbl.Attach (Language, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Source code") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label SourceCode = new Gtk.Label (src.code);
			SourceCode.ModifyFont (fnt);
			SourceCode.Wrap = true;
			SourceCode.Xalign = 0;
			SourceCode.Yalign = 0;
			SourceCode.Selectable = true;
			tbl.Attach (SourceCode, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			SourceCodeType src = (SourceCodeType) entry.data;
			return src.code;
		}
		
		
		public override object GetData ()
		{
			SourceCodeType src = new SourceCodeType ();
			src.language = EditLanguage.Text;
			src.code     = EditCode.Buffer.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", src.language);
			
			return src;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			SourceCodeType src = (SourceCodeType) entry.data;
			return Engine.Text.Search (query, src.language) || Engine.Text.Search (query, src.code);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditLanguage != null) && (EditCode != null) &&
				(EditLanguage.Text != null) && (EditCode.Buffer.Text != "");
		}
	}
}
