/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class WebAddress : DataType {
		private struct WebAddressType {
			public string name;
			public string url;
		}
	
		private Gtk.Entry EditName;
		private Gtk.Entry EditURL;
		
		
		public override string Name {
			get { return "webaddress"; }
		}
		
		public override string Title {
			get { return Catalog.GetString ("Web address"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a web address"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditName; }
		}

		
		public override Gtk.Widget FocusWidget {
			get { return EditName; }
		}


		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			WebAddressType addr = new WebAddressType ();
			addr.name = reader.GetAttribute ("name");
			addr.url  = reader.ReadString ();
			entry.type = Name;
			entry.data = addr;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":name", addr.name);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":url", addr.url);
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			WebAddressType addr = (WebAddressType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("name", addr.name);
				writer.WriteString (addr.url);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Name"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.50;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditName = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditName, "data:" + Name + ":name");
			tbl.Attach (EditName, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("URL"));
			lbl2.Xalign = 1;
			lbl2.Yalign = (float) 0.50;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditURL = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditURL, "data:" + Name + ":url");
			tbl.Attach (EditURL, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditName.Text = "";
				EditURL.Text  = content;
			} else {
				WebAddressType addr = (WebAddressType) entry.data;
				EditName.Text = addr.name;
				EditURL.Text = addr.url;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditName != null) && (EditURL != null)) {
				EditName.Changed += hdlr;
				EditURL.Changed  += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			WebAddressType addr = (WebAddressType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Name") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Title  = new Gtk.Label (addr.name);
			Title.Xalign     = 0;
			Title.Yalign     = (float) 0.50;
			Title.Selectable = true;
			tbl.Attach (Title, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("URL") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Selectable = true;
			lbl2.Yalign    = (float) 0.50;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Incollector.GUI.Widgets.LinkLabel URL = new Incollector.GUI.Widgets.LinkLabel (addr.url);
			tbl.Attach (URL, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			WebAddressType addr = (WebAddressType) entry.data;
			return addr.name + " (" + addr.url + ")";
		}
		
		
		public override object GetData ()
		{
			WebAddressType addr = new WebAddressType ();
			addr.name = EditName.Text;
			addr.url  = EditURL.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":name", addr.name);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":url", addr.url);
			
			return addr;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			WebAddressType addr = (WebAddressType) entry.data;
			return Engine.Text.Search (query, addr.name) || Engine.Text.Search (query, addr.url);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditName != null) && (EditURL != null) &&
				(EditName.Text != "") && (EditURL.Text != "");
		}
	}
}
