/*
 *  Types/Note.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class Word : DataType {
		private struct WordType {
			public string language;
			public string word;
			public string translation;
		}
	
	
		private Gtk.Entry EditLanguage;
		private Gtk.Entry EditWord;
		private Gtk.Entry EditTranslation;
		
		
		public override string Name {
			get { return "word"; }
		}
		
		
		public override string Title {
			get { return Catalog.GetString ("Word"); }
		}
		
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a word"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return EditLanguage; }
		}
		
		
		public override Gtk.Widget FocusWidget {
			get { return EditLanguage; }
		}


		public override Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry)
		{
			WordType wrd = new WordType ();
			wrd.language    = reader.GetAttribute ("language");
			wrd.word        = reader.GetAttribute ("word");
			wrd.translation = reader.ReadString ();
			entry.type = Name;
			entry.data = wrd;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", wrd.language);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":word", wrd.word);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":translation", wrd.translation);
			
			return entry;
		}
		
		
		public override void WriteXml (XmlTextWriter writer, Engine.EntryStruct entry)
		{
			WordType wrd = (WordType) entry.data;
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("language", wrd.language);
				writer.WriteAttributeString ("word", wrd.word);
				writer.WriteString (wrd.translation);
			writer.WriteEndElement ();
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry)
		{
			return EntryEditWidget (entry, "");
		}
		
		
		public override Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content)
		{
			Gtk.Table tbl = new Gtk.Table (2, 3, false);
			
			Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Language"));
			lbl1.Xalign = 1;
			lbl1.Yalign = (float) 0.5;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditLanguage = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditLanguage, "data:" + Name + ":language");
			tbl.Attach (EditLanguage, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			
			Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Word"));
			lbl2.Xalign = 1;
			lbl2.Yalign = (float) 0.5;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditWord = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditWord, "data:" + Name + ":word");
			tbl.Attach (EditWord, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			
			Gtk.Label lbl3 = new Gtk.Label (Catalog.GetString ("Translation"));
			lbl2.Xalign = 1;
			lbl2.Yalign = (float) 0.5;
			tbl.Attach (lbl3, 0, 1, 2, 3, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			EditTranslation = new Gtk.Entry ();
			Incollector.Engine.Autocompletion.Assign (EditTranslation, "data:" + Name + ":translation");
			tbl.Attach (EditTranslation, 1, 2, 2, 3,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			
			if (entry.data == null) {
				EditLanguage.Text = "";
				EditWord.Text = content;
				EditTranslation.Text = "";
			} else {
				WordType wrd = (WordType) entry.data;
				EditLanguage.Text = wrd.language;
				EditWord.Text = wrd.word;
				EditTranslation.Text = wrd.translation;
			}
			
			return tbl;
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			if ((EditLanguage != null) && (EditWord != null) && (EditTranslation != null)) {
				EditLanguage.Changed    += hdlr;
				EditWord.Changed        += hdlr;
				EditTranslation.Changed += hdlr;
			}
		}
		
		
		public override Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry)
		{
			WordType wrd = (WordType) entry.data;
			
			Gtk.Table tbl = new Gtk.Table (2, 2, false);
			
			Gtk.Label lbl1 = new Gtk.Label ();
			lbl1.UseMarkup = true;
			lbl1.Markup    = "<b>" + Catalog.GetString ("Language") + "</b>";
			lbl1.Xalign    = 1;
			tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Language  = new Gtk.Label (wrd.language);
			Language.Xalign     = 0;
			Language.Yalign     = 0;
			Language.Selectable = true;
			tbl.Attach (Language, 1, 2, 0, 1,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			Gtk.Label lbl2 = new Gtk.Label ();
			lbl2.UseMarkup = true;
			lbl2.Markup    = "<b>" + Catalog.GetString ("Word") + "</b>";
			lbl2.Xalign    = 1;
			lbl2.Yalign    = 0;
			tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Word  = new Gtk.Label (wrd.word);
			Word.Xalign     = 0;
			Word.Yalign     = 0;
			Word.Selectable = true;
			tbl.Attach (Word, 1, 2, 1, 2,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
						
			Gtk.Label lbl3 = new Gtk.Label ();
			lbl3.UseMarkup = true;
			lbl3.Markup    = "<b>" + Catalog.GetString ("Translation") + "</b>";
			lbl3.Xalign    = 1;
			lbl3.Yalign    = 0;
			tbl.Attach (lbl3, 0, 1, 2, 3, AttachOptions.Fill,
						AttachOptions.Fill, 3, 3);
			
			Gtk.Label Translation  = new Gtk.Label (wrd.translation);
			Translation.Xalign     = 0;
			Translation.Yalign     = 0;
			Translation.Selectable = true;
			tbl.Attach (Translation, 1, 2, 2, 3,
						AttachOptions.Expand | AttachOptions.Fill,
						AttachOptions.Expand | AttachOptions.Fill, 3, 3);
			
			tbl.ShowAll ();
			
			return tbl;
		}
		
		
		public override string GetTitle (Engine.EntryStruct entry)
		{
			WordType wrd = (WordType) entry.data;
			return wrd.word + " - " + wrd.translation;
		}
		
		
		public override object GetData ()
		{
			WordType wrd = new WordType ();
			wrd.language    = EditLanguage.Text;
			wrd.word        = EditWord.Text;
			wrd.translation = EditTranslation.Text;
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":language", wrd.language);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":word", wrd.word);
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":translation", wrd.translation);
			
			return wrd;
		}
		
		
		public override bool Search (Engine.EntryStruct entry, string query)
		{
			WordType wrd = (WordType) entry.data;
			return Engine.Text.Search (query, wrd.language) || Engine.Text.Search (query, wrd.word) || Engine.Text.Search (query, wrd.translation);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (EditLanguage != null) && (EditWord != null) && (EditTranslation != null) &&
				(EditLanguage.Text != "") && (EditWord.Text != "") && (EditTranslation.Text != "");
		}
	}
}
